/*
############################################ 
# BibLendServlet.java			   #
# 					   #	
# Servlet zur Verwaltung der Verleih Daten #
# 					   #
# Jonas Pohlandt 2002			   #
############################################
*/

import javax.servlet.*;
import javax.servlet.http.*;

import java.util.*;
import java.util.regex.*;

import java.io.*;


public class BibLendServlet extends HttpServlet {

// ## private Variablen ##

    private String xmlpath  = BibSearchRessourceLocator.getXMLPath();
    private String xslpath  = BibSearchRessourceLocator.getXSLPath();
    private String htmlpath = BibSearchRessourceLocator.getHTMLPath();
    private String logpath = BibSearchRessourceLocator.getLogPath();
    
// diese Methode wird vom Servlet Container aufgerufen wenn auf die mit dem Servlet verbundene URL
// ein HTTP Request vom Typ GET stattfindet

    protected void doGet(HttpServletRequest req, HttpServletResponse res)
            throws ServletException, IOException {
	
	// ContentType fr den Browser setzen.
        res.setContentType("text/html");

	// res.getWriter gibt den Writer zurck mit dem die Antwort auf den HTTP Request geschrieben wird
        PrintWriter out = res.getWriter();

	// Die Antwort wird aus einem File gelesen und dann an out geschickt. So knnen nderungen an den Eingabe-
	// masken gemacht werden, ohne dass die Servlet Quellen verndert werden mssen.
        try {
            File       f    = new File(htmlpath + "BibLendServlet.html");
            FileReader fr   = new FileReader(f);
            // char Array von der Grsse der Datei Anlegen
            char[]     cbuf = new char[(int) f.length()];
            
            // i ist in diesem Fall nutzlos, die readMethode gibt jedoch die Zahl gelesener Bytes zurck
            int        i    = fr.read(cbuf);

            // Jetzt werden die aus der Datei gelesenen Daten an den Browser des Benutzers geschickt.
            out.print(cbuf);
            out.close();
        
        // mgliche Exceptions fangen    
        } catch (IOException e) {
            out.println(
                "Konnte HTML Quelldatei nicht &ouml;ffnen. Bitte Administrator verst&auml;ndigen.");
            out.close();
        }
    }

// diese Methode wird vom Servlet Container aufgerufen wenn auf die mit dem Servlet verbundene URL
// ein HTTP Request vom Typ POST stattfindet

    protected void doPost(HttpServletRequest req, HttpServletResponse res)
            throws ServletException, IOException {
        
        // Der Parameter "requester" enthlt die Angabe, "wer" den HTTP Request gettigt hat
        // Mgliche Werte: "search" -> Normale Suchmaske
        //		   "lend"   -> Verleihstatus ndern Maske
        String   requester = req.getParameter("requester");
        
        // Der Parameter "button" enthlt die Angabe, welcher Knopf gedrckt wurde. 
        // Er ist nur bei der Verleihstatus ndern Maske von Bedeutung da dort ein Formular 2 Knpfe enthlt.
        // Mgliche Werte: "Zurueckgegeben"
        //		   "Uebernehmen"
        String   button    = req.getParameter("button");
        
        // File Objekte erzeugen
        File     datafile  = new File(xmlpath + "biblist.xml");
        File     lendfile  = new File(xmlpath + "lendlist.xml");
        File     stylesheet;
        File 	 logfile = new File(logpath + "bibsearch.log");
        
        String   outstring;
        
        // BibList aus der XML Datei erstellen
        BibList  biblist       = new BibList(datafile);
        
        // resultbiblist wird spter mit den gefundenen BibItems gefllt. Sie wird zur Visualisierung
        // der Ergebnisse bentigt.
        BibList  resultbiblist = new BibList();
        
        // LendList aus der XML Datei erstellen
        LendList lendlist      = new LendList(lendfile);

	// ContentType + Header fr den Browser setzen.
        res.setContentType("text/html");
        res.setHeader("pragma", "no-cache");
	
	// res.getWriter gibt den Writer zurck mit dem die Antwort auf den HTTP Request geschrieben wird
        PrintWriter out = res.getWriter();

        if (requester == null) {
            requester = "unauthorized";
        }
        
        // Normale Suche
        if (requester.equals("search")) {
        
            // BibList mit allen gematchten BibItems erstellen
            resultbiblist = biblist.search(req, lendlist);

            out.print(
                "<HTML><HEAD><TITLE>Suchen in den IBR Bibliotheksdaten: Ergebnis</TITLE></HEAD><BODY>");

            if (resultbiblist.isEmpty()) {
                out.print("<H3>Nichts gefunden</H3>");
            } else {
                out.print(
                    "<H3>Folgende Datens&auml;tze wurden gefunden:</H3>");

                stylesheet = new File(xslpath + "BibLend.www.xsl");

                // transform erzeugt ein DOM Document aus der BibList und transformiert 
                // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
                // String zurckgegeben.
                outstring  = resultbiblist.transform(stylesheet);

                out.print(outstring);
            }

            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
            
        // Ein BibItem wird zurckgegeben    
        } else if (requester.equals("lend")
                   && button.equals("Zurueckgegeben")) {
            
            // uniqueid enthlt den Wert des key Attributes
            String uniqueid = req.getParameter("uniqueid");
            
            // Logging
	    FileWriter fw = new FileWriter(logfile, true);
	    GregorianCalendar today = new GregorianCalendar();
	    String now = "\n" + today.get(Calendar.DAY_OF_MONTH) + "-" + today.get(Calendar.MONTH) + "-" + today.get(Calendar.YEAR) + " ";
	    now += today.get(Calendar.HOUR_OF_DAY) + ":" + today.get(Calendar.MINUTE) + ":" + today.get(Calendar.SECOND) + " -> ";
	    fw.write(now + "*Rckgabe* key: " + req.getParameter("uniqueid"));
	    fw.write(now + "*Rckgabe* Name: " + req.getParameter("lentbyname"));
	    fw.write(now + "*Rckgabe* MatrikelNr: " + req.getParameter("lentbymatno"));
	    fw.write("\n###");
	    fw.close();
	    
	    // LendItem mit den Verleih Informationen ber das BibItem entfernen.
	    lendlist.removeLendItem(uniqueid);
            
            // Speichern
            lendlist.toFile(lendfile);
            
            // Als ergebnis sieht man dann das BibItem bei dem die Verleihinformation entfernt wurde.
            resultbiblist.addBibItem(biblist.getBibItem(uniqueid));
            out.print(
                "<HTML><HEAD><TITLE>&Auml;ndern eines Verleihstatus: Ergebnis</TITLE></HEAD><BODY>");
            out.print(
                "<H2>Folgendes Werk wurde zur&uuml;ckgegeben, der Verleiheintrag wurde gel&ouml;scht:</H2>");

            stylesheet = new File(xslpath + "BibSearch.www.xsl");

            // transform erzeugt ein DOM Document aus der BibList und transformiert 
            // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
            // String zurckgegeben.
            outstring  = resultbiblist.transform(stylesheet);

            out.print(outstring);
            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
            
        // Verleihinformation eines BibItems (also das dazugehrige LendItem) verndern.
        } else if (requester.equals("lend") && button.equals("Uebernehmen")) {
            
            out.print(
                "<HTML><HEAD><TITLE>&Auml;ndern eines Verleihstatus: Ergebnis</TITLE></HEAD><BODY>");
            
            String lenddate = req.getParameter("lenddate");
            GregorianCalendar today = new GregorianCalendar();
            
            // Falls in der Eingabe Maske ein falsches Datumsformat verwendet wurde:
            if (!(checkDate(lenddate, false))){
            	
            	out.print("<H3>Das angegebene Verleih-Datum entspricht nicht dem geforderten Format oder liegt in der Zukunft.</H3>");
            	out.print("Eingabe: " + lenddate + "<BR />Gefordert: TT MM JJJJ (z.B. 14 04 2002)<BR />");
            	out.print("Heute ist der " + today.get(Calendar.DAY_OF_MONTH) + ". " + today.get(Calendar.MONTH) + ". " + today.get(Calendar.YEAR) + " <BR />&nbsp;<BR />");
            	out.print("<I>Benutzen Sie bitte den Zur&uuml;ckknopf des Browsers um die Eingaben nicht zu verlieren.</I>");
            } else {
            	// uniqueid enthlt den Wert des key Attributes
            	String uniqueid = req.getParameter("uniqueid");
            	
            	// logging
            	FileWriter fw = new FileWriter(logfile, true);
	        String now = "\n" + today.get(Calendar.DAY_OF_MONTH) + "-" + today.get(Calendar.MONTH) + "-" + today.get(Calendar.YEAR) + " ";
	        now += today.get(Calendar.HOUR_OF_DAY) + ":" + today.get(Calendar.MINUTE) + ":" + today.get(Calendar.SECOND) + " -> ";
	        fw.write(now + "*nderung* key: " + req.getParameter("uniqueid"));
	        fw.write(now + "*nderung* Name: " + req.getParameter("lentbyname"));
	        fw.write(now + "*nderung* MatrikelNr: " + req.getParameter("lentbymatno"));
	        fw.write(now + "*nderung* Ausgeliehen am: " + req.getParameter("lenddate"));
	        fw.write("\n###");
	        fw.close();
            	
            	// LendItem mit den Verleih Informationen ber das BibItem entfernen.            	
                lendlist.removeLendItem(uniqueid);
                
                // neues LendItem aus dem HTTP Request erstellen
                LendItem lenditem = new LendItem(req);
                
                //neues LendItem der LendList hinzufgen;
                lendlist.addLendItem(lenditem);
                
                // Speichern
                lendlist.toFile(lendfile);
                
                // Zur Visualisierung wird dem BibItem jetzt noch die Verleih Information hinzugefgt.
                // Diese Information wird nie gespeichert sondern lediglich zur Visualisierung genutzt.
                // Die eigentliche Verleihinformation ist im LendItem gespeichert.
                BibItem bibitem = biblist.getBibItem(uniqueid);
                bibitem.addLendInfo(lenditem);
                resultbiblist.addBibItem(bibitem);
                
                out.print(
                    "<H2>Der Verleihstatus wurde wie folgt ge&auml;ndert:</H2>");
    
                stylesheet = new File(xslpath + "BibLendSearch.www.xsl");
                
		// transform erzeugt ein DOM Document aus der BibList und transformiert 
                // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
                // String zurckgegeben.                
                outstring  = resultbiblist.transform(stylesheet);
    
                out.print(outstring);
            }
            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
        } else {
            out.print("Unauthorisiert!<BR />\n");
            out.print("Requester: " + requester + "<BR />\n");
            out.print("Button: " + button);
        }
        
        // Jetzt werden die mit out.print() geschriebenen Zeichen an den Browser des Benutzers geschickt.
        out.close();
    }
    
// Diese Methode prft eine Zeichenkette, ob sie die Form TT MM JJJJ hat und ob sie gltige Werte
// (im Sinne eines gregorianischen Kalenders) hat. 
//
// Parameter:	datestring: die zu prfende Zeichenkette
// 		lenient: gibt an, ob auch zuknftige Daten erlaubt sind. true: auch zuknftige Daten erlaubt
//
// Rckgabewert: true wenn das Datum akzeptiert wird, sonst false.
    private boolean checkDate(String datestring, boolean lenient) {

        int               year, month, day;
        GregorianCalendar gc      = new GregorianCalendar();
        GregorianCalendar now     = new GregorianCalendar();
        boolean           okay    = false;
        Pattern           pattern =
        	Pattern.compile("([0-9]{2})\\s([0-9]{2})\\s([0-9]{4})");
        Matcher           matcher = pattern.matcher(datestring);

        if (!matcher.matches()) {
            return okay;
        }

        day   = Integer.parseInt(matcher.group(1));
        month = Integer.parseInt(matcher.group(2));
        year  = Integer.parseInt(matcher.group(3));

        if (year > 1950) {
            if ((month == 1) || (month == 3) || (month == 5) || (month == 7)
                    || (month == 8) || (month == 10) || (month == 12)) {
                if ((1 <= day) || (day <= 31)) {
                    okay = true;
                }
            }

            if ((month == 4) || (month == 6) || (month == 9)
                    || (month == 11)) {
                if ((1 <= day) || (day <= 30)) {
                    okay = true;
                }
            }

            if (month == 2) {
                if (gc.isLeapYear(year)) {
                    if ((1 <= day) || (day <= 29)) {
                        okay = true;
                    }
                } else {
                    if ((1 <= day) || (day <= 28)) {
                        okay = true;
                    }
                }
            }
        }

        if (lenient) {
            return okay;
        }

        gc.set(year, month, day);

        if (gc.after(now)) {
            okay = false;
        }

        return okay;
    }

    public String getServletInfo() {
        return "BibLendServlet IBR@TU-BS (c)2002";
    }
    
// Diese Methode wird lediglich zu debug Zwecken verwendet und wird im Normalzustand nie aufgerufen.
    void debug(HttpServletRequest req) {
        System.out.println("---");
        for (Enumeration e = req.getParameterNames(); e.hasMoreElements(); ) {
            String s = (String) e.nextElement();
            System.out.println(s + ": " + req.getParameter(s));
        }
    }
}
