/*
################################################ 
# BibMaintainServlet.java		       #
# 					       #	
# Servlet zur Verwaltung der Bibliotheks Daten #
# 					       #
# Jonas Pohlandt 2002			       #
################################################
*/
import javax.servlet.*;
import javax.servlet.http.*;

import java.util.Enumeration;
import java.io.*;
import java.lang.Integer;

public class BibMaintainServlet extends HttpServlet {

// ## private Variablen ##

    private String xmlpath  = BibSearchRessourceLocator.getXMLPath();
    private String xslpath  = BibSearchRessourceLocator.getXSLPath();
    private String htmlpath = BibSearchRessourceLocator.getHTMLPath();

// diese Methode wird vom Servlet Container aufgerufen wenn auf die mit dem Servlet verbundene URL
// ein HTTP Request vom Typ GET stattfindet

    protected void doGet(HttpServletRequest req, HttpServletResponse res)
            throws ServletException, IOException {
	
	// ContentType fr den Browser setzen.
        res.setContentType("text/html");
	
	// res.getWriter gibt den Writer zurck mit dem die Antwort auf den HTTP Request geschrieben wird
        PrintWriter out = res.getWriter();
	
	// Die Antwort wird aus einem File gelesen und dann an out geschickt. So knnen nderungen an den Eingabe-
	// masken gemacht werden, ohne dass die Servlet Quellen verndert werden mssen.
        try {
            File       f    = new File(htmlpath + "BibMaintainServlet.html");
            FileReader fr   = new FileReader(f);
            
            // char Array von der Grsse der Datei Anlegen
            char[]     cbuf = new char[(int) f.length()];
            
            // i ist in diesem Fall nutzlos, die readMethode gibt jedoch die Zahl gelesener Bytes zurck
            int        i    = fr.read(cbuf);
		
	    // Jetzt werden die aus der Datei gelesenen Daten an den Browser des Benutzers geschickt.
            out.print(cbuf);
            out.close();
            
        // mgliche Exceptions fangen    
        } catch (IOException e) {
            out.println(
                "Konnte HTML Quelldatei nicht &ouml;ffnen. Bitte Administrator verst&auml;ndigen.");
            out.close();
        }
    }

// diese Methode wird vom Servlet Container aufgerufen wenn auf die mit dem Servlet verbundene URL
// ein HTTP Request vom Typ POST stattfindet

    protected void doPost(HttpServletRequest req, HttpServletResponse res)
            throws ServletException, IOException {

        String outstring;
        
        // Der Parameter "requester" enthlt die Angabe, "wer" den HTTP Request gettigt hat
        // Mgliche Werte: "search" -> Normale Suchmaske
        //		   "add"    -> BibItem hinzufgen
        //		   "update" -> BibItem ndern
        String requester = req.getParameter("requester");

        if (requester == null) {
            requester = "unauthorized";
        }

	// Der Parameter "button" enthlt die Angabe, welcher Knopf gedrckt wurde. 
        // Mgliche Werte: "Hinzufuegen" -> 	eingegebene Werte fr neues BibItem prfen, key generieren
        //				 	und dem User zur Besttigung zeigen
        //		   "Bestaetigen"  ->	BibItem tatsaechlich hinzufuegen
        String button = req.getParameter("button");

        if (button == null) {
            button = "unauthorized";
        }
	
	// File Objekte erzeugen
        File    datafile = new File(xmlpath + "biblist.xml");
        File    stylesheet;
        
        // BibList aus der XML Datei erstellen
        BibList biblist       = new BibList(datafile);
        
        // resultbiblist wird spter mit den gefundenen BibItems gefllt. Sie wird zur Visualisierung
        // der Ergebnisse bentigt.
        BibList resultbiblist = new BibList();

	// ContentType + Header fr den Browser setzen.
        res.setContentType("text/html");
        res.setHeader("pragma", "no-cache");
	
	// res.getWriter gibt den Writer zurck mit dem die Antwort auf den HTTP Request geschrieben wird
        PrintWriter out = res.getWriter();

        if (requester == null) {
            requester = "unauthorized";
        }
	
	// Normale Suche
        if (requester.equals("search")) {
        	
            // BibList mit allen gematchten BibItems erstellen
            resultbiblist = biblist.search(req);

            out.print(
                "<HTML><HEAD><TITLE>Suchen in den IBR Bibliotheksdaten: Ergebnis</TITLE></HEAD><BODY>");

            if (resultbiblist.isEmpty()) {
                out.print("<H3>Nichts gefunden</H3>");
            } else {
                out.print(
                    "<H3>Folgende Datens&auml;tze wurden gefunden:</H3>");
                stylesheet = new File(xslpath + "BibMaintainUpdate.www.xsl");
                
                // transform erzeugt ein DOM Document aus der BibList und transformiert 
                // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
                // String zurckgegeben.
                outstring  = resultbiblist.transform(stylesheet);

                out.print(outstring);
            }

            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
        
        // Ein neues BibItem soll hinzugefgt werden. Es wird ein eindeutiger Schlssel erzeugt, den der Benutzer
        // in diesem Schritt prfen soll. 
        } else if (requester.equals("add")
                   && button.equals("Hinzufgen")) {
            // Aus den eingegebene Werten ein neues BibItem erstellen
            BibItem bibitem = new BibItem(req, biblist);
            
            // Ergebnis BibList mit diesem einem BibItem fllen
            resultbiblist.addBibItem(bibitem);
            out.print(
                "<HTML><HEAD><TITLE>Verwaltung der IBR Bibliotheksdaten: Datensatz &uuml;berpr&uuml;fen</TITLE></HEAD><BODY>");

            stylesheet = new File(xslpath + "BibMaintainAdd.www.xsl");
            
            // transform erzeugt ein DOM Document aus der BibList und transformiert 
            // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
            // String zurckgegeben.
            outstring  = resultbiblist.transform(stylesheet);

            out.print(
                "<H2>Folgender Datensatz soll hinzugef&uuml;gt werden: (bitte Feld \"key\" pr&uuml;fen)</H2>");
            out.print(outstring);
            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
            
        // Nachdem der User seine Eingaben und das "key" Feld geprft hat soll der Datensatz jetzt tatsaechlich
        // Hinzugefgt werden
        } else if (requester.equals("add") && button.equals("Bestaetigen")) {

            String uniqueid = req.getParameter("uniqueid");
            
            // Aus den eingegebene Werten ein neues BibItem erstellen
            BibItem bibitem = new BibItem(req, uniqueid);
            
            // Wenn z.B. mehrere Personen gleichzeitig Daten eingeben, kann es passieren, dass der generierte key nicht
            // mehr eindeutig ist. Dieser Fall wird hier abgefangen.
            if (biblist.getBibItem(uniqueid) != null) {
            	out.print(
                    "<HTML><HEAD><TITLE>Verwaltung der IBR Bibliotheksdaten: Datensatz konnte nicht hinzugef&uuml;gt werden</TITLE></HEAD><BODY>");
                out.print("<P>Der Schlssel <B>" + uniqueid  + "</B> exisitiert bereits!</P>");
                out.print("Bitte benutzen Sie den Zur&uuml;ckknopf Ihres Browsers um den key zu &auml;ndern</BODY></HTML>");
            } else {    
    	        // BibList um das neue BibItem erweitern
                biblist.addBibItem(bibitem);
                // Datei Speichern
                biblist.toFile(datafile);
                // Zur Visualisierung das neue BibItem in die Ergebnisliste schreiben
                resultbiblist.addBibItem(bibitem);
                out.print(
                    "<HTML><HEAD><TITLE>Verwaltung der IBR Bibliotheksdaten: Datensatz wurde hinzugef&uuml;gt</TITLE></HEAD><BODY>");
    
                stylesheet = new File(xslpath + "BibSearch.www.xsl");
                
                // transform erzeugt ein DOM Document aus der BibList und transformiert 
                // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
                // String zurckgegeben.
                outstring  = resultbiblist.transform(stylesheet);
    
                out.print("<H2>Folgender Datensatz wurde hinzugef&uuml;gt:</H2>");
                out.print(outstring);
                out.print("<HR /><A HREF=\"");
                out.print(req.getRequestURI());
                out.print("\">Zur&uuml;ck</A></BODY</HTML>");
            }
         
        // Nach der Suche im Maintain Servlet kann man gefudenen BibItems ndern oder
        // Lschen. Hier wird das Lschen erledigt. 
        } else if (requester.equals("update") && button.equals("Loeschen")) {
            String uniqueid = req.getParameter("uniqueid");

            // BibItem aus der BibList lschen
            biblist.removeBibItem(uniqueid);
            
            // Datei speichern
            biblist.toFile(datafile);

	    // Zur Visualisierung das gelschte BibItem in die Ergebnisliste schreiben
            BibItem bibitem = new BibItem(req, uniqueid);
            resultbiblist.addBibItem(bibitem);
            out.print(
                "<HTML><HEAD><TITLE>Verwaltung der IBR Bibliotheksdaten: Datensatz wurde gel&ouml;scht</TITLE></HEAD><BODY>");

            stylesheet = new File(xslpath + "BibSearch.www.xsl");
            
            // transform erzeugt ein DOM Document aus der BibList und transformiert 
            // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
            // String zurckgegeben.
            outstring  = resultbiblist.transform(stylesheet);

            out.print("<H2>Folgender Datensatz wurde gel&ouml;scht:</H2>");
            out.print(outstring);
            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
            
        // Nach der Suche im Maintain Servlet kann man gefudenen BibItems ndern oder
        // Lschen. Hier wird das ndern erledigt. 
        } else if (requester.equals("update")
                   && button.equals("Uebernehmen")) {
            String  uniqueid = req.getParameter("uniqueid");
            BibItem bibitem  = new BibItem(req, uniqueid);
		
            // BibItem mit den urpsrnglichen Daten aus der BibList lschen
            biblist.removeBibItem(uniqueid);
            
            // BibItem mit genderten Daten hinzufgen
            biblist.addBibItem(bibitem);
            
            // Datei speichern
            biblist.toFile(datafile);
            
            // Zur Visualisierung das aktualisierte BibItem in die Ergebnisliste schreiben
            resultbiblist.addBibItem(bibitem);
            out.print(
                "<HTML><HEAD><TITLE>&Auml;ndern von den IBR Bibliotheksdaten: Ergebnis</TITLE></HEAD><BODY>");

            stylesheet = new File(xslpath + "BibSearch.www.xsl");
            
            // transform erzeugt ein DOM Document aus der BibList und transformiert 
            // dieses dann per XSLT mit dem angegebenen Stylesheet. Das Ergebnis wird als
            // String zurckgegeben.
            outstring  = resultbiblist.transform(stylesheet);

            out.print(
                "<H2>Der Datensatz wurde wie folgt ge&auml;ndert:</H2>");
            out.print(outstring);
            out.print("<HR /><A HREF=\"");
            out.print(req.getRequestURI());
            out.print("\">Zur&uuml;ck</A></BODY</HTML>");
        } else {	
            out.print("Unauthorisiert!<BR />\n");
            out.print("Requester: " + requester + "<BR />\n");
            out.print("Button: " + button);
        }

	// Jetzt werden die mit out.print() geschriebenen Zeichen an den Browser des Benutzers geschickt.
        out.close();
    }

    public String getServletInfo() {
        return "BibSearchServlet IBR@TU-BS (c)2002";
    }

// Diese Methode wird lediglich zu debug Zwecken verwendet und wird im Normalzustand nie aufgerufen.
    void debug(HttpServletRequest req) {
        for (Enumeration e = req.getParameterNames(); e.hasMoreElements(); ) {
            String s = (String) e.nextElement();
            System.out.println(s + ": " + req.getParameter(s));
        }
    }
}

