/*
############################################## 
# BibSearchServlet.java		             #
# 					     #	
# Servlet zur Suche in den Bibliotheks Daten #
# 					     #
# Jonas Pohlandt 2002			     #
##############################################
*/
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.w3c.dom.Document;
import java.util.Enumeration;
import java.util.Hashtable;

public class BibSearchServlet extends HttpServlet {

    private Hashtable biblists = new Hashtable();

    // diese Methode wird vom Servlet Container aufgerufen wenn auf
    // die mit dem Servlet verbundene URL ein HTTP Request vom Typ GET
    // stattfindet
    protected void doGet(HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {
	
	ServletContext c = getServletContext();

        BibList resultbiblist = new BibList();

	// res.getWriter gibt den Writer zurck mit dem die Antwort
	// auf den HTTP Request geschrieben wird
        PrintWriter out = res.getWriter();
	
	// ContentType + Header setzen.
	res.setContentType("text/html");

	File stylesheet = new File(c.getInitParameter("xslPath") +
				   "BibSearchServlet-www.xsl");

	// transform erzeugt ein DOM Document aus der BibList
	// und transformiert dieses dann per XSLT mit dem
	// angegebenen Stylesheet. Das Ergebnis wird als
	// String zurckgegeben.
	String outstring = resultbiblist.transform(stylesheet);
	out.print(outstring);

        out.close();
    }

    // diese Methode wird vom Servlet Container aufgerufen wenn auf
    // die mit dem Servlet verbundene URL ein HTTP Request vom Typ
    // POST stattfindet
    protected void doPost(HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException {
	
	BibList biblist;

	ServletContext c = getServletContext();

        // Der Parameter "requester" enthlt die Angabe, "wer" den
        // HTTP Request gettigt hat.
        // Mgliche Werte: "www" -> Anfrage ber das WebInterface
        //		   "cli" -> Anfrage ber das CLI
        String requester = req.getParameter("requester");

        if (requester == null) {
            requester = "unauthorized";
        }

        String  lendinfo      = req.getParameter("lendinfobool");

        BibList resultbiblist = new BibList();

	if (!biblists.containsKey(req.getParameter("database"))) {
	    log("Parsing " + req.getParameter("database") + ".");
	    File datafile = new File(c.getInitParameter("xmlPath") +
				     req.getParameter("database"));
	    biblist = new BibList(this, datafile);
	    biblists.put(req.getParameter("database"), biblist);
	    log("Parsing finished (" + biblist.size() + 
		" entries in database #" + biblists.size() + ").");
	} else {
	    biblist = (BibList)biblists.get(req.getParameter("database"));
	}

	// Falls Verleihinformationen mit angezeigt werden sollen:
        if (lendinfo != null) {
            File     lendfile = new File(c.getInitParameter("xmlPath") +
					 "lend-" + 
					 req.getParameter("database"));
            LendList lendlist = new LendList(lendfile);
	    log("Searching for (" + req.getParameter("title") + 
		"," + req.getParameter("author") + ",...)" +
		" in " + req.getParameter("database") +
		" via " + req.getParameter("requester") + 
		" with lend info.");
            resultbiblist = biblist.search(this, req, lendlist);
	    log("Search returned " + resultbiblist.size() +
		" entries out of " + biblist.size() + " entries.");
        } else {
	    log("Searching for (" + req.getParameter("title") + 
		"," + req.getParameter("author") + ",...)" +
		" in " + req.getParameter("database") +
		" via " + req.getParameter("requester") + 
		" without lend info.");
            resultbiblist = biblist.search(this, req);
	    log("Search returned " + resultbiblist.size() +
		" entries out of " + biblist.size() + " entries.");
        }
	
	// res.getWriter gibt den Writer zurck mit dem die Antwort
	// auf den HTTP Request geschrieben wird
        PrintWriter out = res.getWriter();
        
        // Falls die Suchanfrage vom WebInterface kommt:
        if (requester.equals("www")) {
	    // ContentType + Header setzen.
	    res.setContentType("text/html");
	    res.setHeader("pragma", "no-cache");

            File stylesheet = new File(c.getInitParameter("xslPath") +
                                       "BibSearchServlet-www.xsl");

	    // transform erzeugt ein DOM Document aus der BibList
	    // und transformiert dieses dann per XSLT mit dem
	    // angegebenen Stylesheet. Das Ergebnis wird als
	    // String zurckgegeben.
	    String outstring = resultbiblist.transform(stylesheet, req);
	    out.print(outstring);
        }

	// Falls die Suchanfrage vom CLI kommt:
	else if (requester.equals("cli")) {
	    // ContentType + Header setzen.
	    res.setContentType("text/plain");
	    res.setHeader("pragma", "no-cache");

            File stylesheet = new File(c.getInitParameter("xslPath") +
				       "BibSearchServlet-cli.xsl");

	    // transform erzeugt ein DOM Document aus der BibList
	    // und transformiert dieses dann per XSLT mit dem
	    // angegebenen Stylesheet. Das Ergebnis wird als
	    // String zurckgegeben.
	    String outstring = resultbiblist.transform(stylesheet);
	    out.print(outstring);
        } else {
            out.println("ERROR: Unknown requestor type.");
        }
        out.close();
    }

    public String getServletInfo() {
        return "BibSearchServlet (c) 2002 " +
	    "Jonas Pohlandt <pohlandt@ibr.cs.tu-bs.de>";
    }
    
    // Diese Methode wird lediglich zu debug Zwecken verwendet und
    // wird im Normalzustand nie aufgerufen.
    void debug(HttpServletRequest req) {

        for (Enumeration e = req.getParameterNames(); e.hasMoreElements(); ) {
            String s = (String) e.nextElement();

            System.out.println(s + ": " + req.getParameter(s));
        }
    }
}

