/*
############################################ 
# LendList.java				   #
# 					   #	
# Stellt ein LendList Element dar	   #
# 					   #
# Jonas Pohlandt 2002			   #
############################################
*/

import javax.servlet.http.HttpServletRequest;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.OutputKeys;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.*;

import java.util.*;
import java.io.*;
import java.lang.Integer;


public class LendList {

    // ## private Variablen ##

    // In dieser Hashtable werden die LendItems aufbewahrt
    // Hashkey: Wert des key Elementes des LendItems
    // Hashwert: Das LendItem selber
    private Hashtable lenditems = new Hashtable();
    
    // ErrorHandler der beim parsen von XML verwendet wird
    MySaxErrorHandler mySaxEH;
    // String            pathToDTD = BibSearchRessourceLocator.getXMLPath()
    //                               + "lendlist.dtd";
    // ## Konstruktoren ##

    // Leerer Konstruktor
    public LendList() {}

    // Erstellt eine LendList aus einem DOM Document
    //
    // Parameter: Das DOM Document aus welchem die LendList erzeugt
    // werden soll.
    public LendList(Document document) {

	// Alle LendItem Elemente aus dem DOM Document in eine
	// NodeList schreiben
        NodeList list = document.getElementsByTagName("lenditem");

	// Jedes LendItem mit dem Wert seines key Elementes als
	// Hashwert in die Hashtable schreiben
        for (int i = 0; i < list.getLength(); i++) {
            Node     node     = list.item(i);    //i-ter LendItem Node
            LendItem lenditem = new LendItem(node);
            lenditems.put(lenditem.getElement("key"), lenditem);
        }
    }

    // Erstellt eine LendList aus einer XML Datei. Dabei wird zuerst
    // die XML Datei in ein DOM document gewandelt, dann verhlt sich
    // der Konstruktor genauso wie der vorhergehende
    //
    // Parameter: Das XML File aus welchem die LendList erzeugt werden
    // soll.
    public LendList(File xmlfile) {

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(true);

        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setErrorHandler(mySaxEH);
            Document document = builder.parse(xmlfile);
            
            // Alle BibItem Elemente aus dem DOM Document in eine
            // NodeList schreiben
            NodeList list     = document.getElementsByTagName("lenditem");
	    
	    // Jedes BibItem mit dem Wert seines key Attributes als
	    // Hashwert in die Hashtable schreiben
            for (int i = 0; i < list.getLength(); i++) {
                Node     node     = list.item(i);    //i-ter LendItem Node
                LendItem lenditem = new LendItem(node);

                lenditems.put(lenditem.getElement("key"), lenditem);
            }
        
        // Mgliche Exceptions fangen
        } catch (SAXException sxe) {
            Exception x = sxe;
            if (sxe.getException() != null) {
                x = sxe.getException();
            }
            x.printStackTrace();
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }

    // Gibt alle keys der Hashtable in Form einer Enumeration zurck.
    public Enumeration keys() {
        return lenditems.keys();
    }

    // Die LendList ist leer, falls die Hashtable leer ist.
    public boolean isEmpty() {
        boolean empty = lenditems.isEmpty();
        return empty;
    }

    // Diese Methode erzeugt ein DOM Document, welches der LendList
    // entspricht.
    //
    // Rckgabewert: Das erzeugte DOM Document
    public Document toDocument() {

        Enumeration            e;
        Document               document = null;
        DocumentBuilderFactory factory  =
            DocumentBuilderFactory.newInstance();
        factory.setValidating(true);

        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            builder.setErrorHandler(mySaxEH);
            document = builder.newDocument();
            Element rootelement =
                (Element) document.createElement("lendlist");
            document.appendChild(rootelement);
            e = lenditems.keys();

            while (e.hasMoreElements()) {
                String   key      = (String) e.nextElement();
                LendItem lenditem = (LendItem) lenditems.get(key);

                document.getDocumentElement()
                    .appendChild(document.importNode(lenditem.toNode(),
                                                     true));
            }
            
        // Mgliche Exceptions fangen
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        }
        return document;
    }
    
    // Methoden fr den direkten Zugriff auf die Hashtable
    public void addLendItem(LendItem lenditem) {
        lenditems.put(lenditem.getElement("key"), lenditem);
    }

    public void removeLendItem(String uniqueid) {
        lenditems.remove(uniqueid);
    }

    public LendItem getLendItem(String uniqueid) {
        return (LendItem) lenditems.get(uniqueid);
    }
    
    // Diese Methode schreibt die LendList im XML Format in eine
    // Datei. Dies geschieht, indem die LendList zuerst in ein DOM
    // Document gewandelt wird und dann ber die von
    // javax.xml.transform zur Verfgung gestellten Methoden in ein
    // StreamResult (in diesem Fall ein File) geschrieben wird.
    //
    // Parameter: Das zu speichernde File
    public void toFile(File file) {
	
	// Document welches spter in das File transformiert wird.
        Document document;
        try {
            // Das Document mit der LendList fllen.
            document = this.toDocument();
            
            // Neuen Transformer per TransformerFactory erstellen.
            TransformerFactory tFactory    = TransformerFactory.newInstance();
            Transformer        transformer = tFactory.newTransformer();
	    
	    // Transformer konfigurieren. Hier wird angegeben, das
            // spter in der XML Datei <!DOCTYPE biblist SYSTEM
            // "[pathToDTD]"> stehen soll
            // transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM,
            //                               pathToDTD);
	    // Source und Result fr den Transformer angeben
            DOMSource    source = new DOMSource(document);
            StreamResult result = new StreamResult(file);

	    // TRansformieren
            transformer.transform(source, result);
        
        // Mgliche Exceptions fangen
        } catch (TransformerConfigurationException tce) {
            System.out.println("\n** Transformer Factory error");
            System.out.println("   " + tce.getMessage());
            Throwable x = tce;
            if (tce.getException() != null) {
                x = tce.getException();
            }
            x.printStackTrace();
        } catch (TransformerException te) {
            System.out.println("\n** Transformation error");
            System.out.println("   " + te.getMessage());
            Throwable x = te;
            if (te.getException() != null) {
                x = te.getException();
            }
            x.printStackTrace();
        }
    }
}
